@@ -1,13 +1,22 @@
 /* vim: set sw=8 ts=8 si et: */
 /*********************************************
 * An implemenation of the STK500 specification (appl. note AVR068) for atmega8
-* Author: Guido Socher, Copyright: GPL
+* * Author: Guido Socher, Copyright: GPL
+*
+* Terminal capabilities for updating SW/HW version:
+* Idea from Florin-Viorel Petrov, www.fonitzu.com/electronics, Date: 28Feb2006
+* Florin-Viorel's version was a bit more fancy than the version implemented
+* here. This implemenation is just small and simple.
+*
 * Copyright: GPL
 **********************************************/
 #include <inttypes.h>
 #include <avr/interrupt.h>
 #include <string.h>
 #include <avr/io.h>
+#include <avr/pgmspace.h>
+#include <stdlib.h>
+#include <avr/eeprom.h>
 #include "uart.h"
 #include "led.h"
 #include "spi.h"
@@ -17,15 +26,24 @@
 
 #define CONFIG_PARAM_BUILD_NUMBER_LOW   0
 #define CONFIG_PARAM_BUILD_NUMBER_HIGH  1
+// be careful with changing HW versions. avrstudio does not like all numbers:
 #define CONFIG_PARAM_HW_VER             2
-// update here our own sw version:
-#define CONFIG_PARAM_SW_MAJOR           2
-#define CONFIG_PARAM_SW_MINOR           2
+// update here our own default sw version:
+#define D_CONFIG_PARAM_SW_MAJOR           2
+#define D_CONFIG_PARAM_SW_MINOR           1
 #define CONFIG_PARAM_VTARGET 50
 #define CONFIG_PARAM_VADJUST 25
 #define CONFIG_PARAM_OSC_PSCALE 2
 #define CONFIG_PARAM_OSC_CMATCH 1
 
+// you can change the CONFIG_PARAM_SW in terminal mode
+#define EEPROM_MAJOR            ((uint8_t*)2)
+#define EEPROM_MINOR            ((uint8_t*)1)
+#define EEPROM_MAGIC            ((uint8_t*)0)
+uint8_t CONFIG_PARAM_SW_MAJOR;
+uint8_t CONFIG_PARAM_SW_MINOR;
+const char terminal_init[] PROGMEM = {"\x1B[0m\x1B[2J\x1B[0;0f"};
+
 #define MSG_IDLE 0
 #define MSG_WAIT_SEQNUM 1
 #define MSG_WAIT_SIZE1 2
@@ -89,7 +107,7 @@ void programcmd(unsigned char seqnum)
                 msg_buf[0] = CMD_SIGN_ON; // 0x01
                 msg_buf[1] = STATUS_CMD_OK; // 0x00
                 msg_buf[2] = 8; //len
-                strcpy(&(msg_buf[3]),"AVRISP_2"); // note: this copied also the null termination
+                strcpy((char *)&(msg_buf[3]),"AVRISP_2"); // note: this copied also the null termination
                 answerlen=11;
                 break;
 
@@ -292,6 +310,7 @@ void programcmd(unsigned char seqnum)
                 // msg_buf[9] poll2
                 // msg_buf[n+10] Data
                 poll_address=0;
+		ci=150;
                 // set a minimum delay
                 if (msg_buf[4] < 3){
                         msg_buf[4]=3;
@@ -381,6 +400,9 @@ void programcmd(unsigned char seqnum)
                                         //increment address
                                         address++;
                                 }
+				if (ci==0){
+					cstatus=STATUS_CMD_TOUT;
+				}
                         }                        
                 }else{
                         //page mode, all modern chips
@@ -578,9 +600,88 @@ void programcmd(unsigned char seqnum)
         
 }
 
-void main(void)
+void terminalmode_next_line(void)
+{
+        uart_sendchar('\x1B');
+        uart_sendchar('E');
+}
+
+void terminalmode(unsigned char iswin)
 {
         unsigned char ch;
+        unsigned char i;
+	// Init terminal
+	uart_sendstr_p(terminal_init);
+	//
+	uart_sendstr_P("AvrUsb500 terminal mode\r\n");
+	i=1;
+	if (iswin){
+		i=2;
+	}
+	uart_sendstr_P("Enter SW Version Major [");
+	utoa(CONFIG_PARAM_SW_MAJOR,(char *)msg_buf, 10);
+	uart_sendstr((char *)msg_buf);
+	uart_sendstr_P("]: ");
+	ch=uart_getchar();
+	if (ch != '\r' && ch !='\n'){
+		uart_sendchar(ch); // echo number back to terminal
+	}
+	if ((ch > '0') && (ch < '9')){
+		CONFIG_PARAM_SW_MAJOR=ch - (unsigned char)'0';
+	}
+	// user just pressed return:
+	if (ch == '\r' || ch == '\n'){
+		i--;
+	}
+	// get \n or \r\n after a number was entered:
+	while(i){
+		ch=uart_getchar();
+		i--;
+	}
+	terminalmode_next_line();
+	uart_sendstr_P("Enter SW Version Minor [");
+	utoa(CONFIG_PARAM_SW_MINOR,(char *)msg_buf, 10);
+	uart_sendstr((char *)msg_buf);
+	uart_sendstr_P("]: ");
+	i=1;
+	if (iswin){
+		i=2;
+	}
+	ch=uart_getchar();
+	if (ch != '\r' && ch !='\n'){
+		uart_sendchar(ch); // echo number back to terminal
+	}
+	if ((ch > '0') && (ch < '9')){
+		CONFIG_PARAM_SW_MINOR=ch - (unsigned char)'0';
+	}
+	// user just pressed return:
+	if (ch == '\r' || ch == '\n'){
+		i--;
+	}
+	// get \n or \r\n after a number was entered:
+	while(i){
+		ch=uart_getchar();
+		i--;
+	}
+	terminalmode_next_line();
+	eeprom_write_byte(EEPROM_MINOR, CONFIG_PARAM_SW_MINOR);
+	eeprom_write_byte(EEPROM_MAJOR, CONFIG_PARAM_SW_MAJOR);
+	eeprom_write_byte(EEPROM_MAGIC, 20);// magic number=data is valid
+	uart_sendstr_P("\r\nOK, my SW version is now: ");
+	utoa(CONFIG_PARAM_SW_MAJOR,(char *)msg_buf, 10);
+	uart_sendstr((char *)msg_buf);
+	uart_sendchar('.');
+	utoa(CONFIG_PARAM_SW_MINOR,(char *)msg_buf, 10);
+	uart_sendstr((char *)msg_buf);
+	uart_sendstr_P("\r\n");
+	uart_sendstr_P("Ready. Just close the terminal. No reset needed.\r\n");
+}
+
+int main(void)
+{
+        unsigned char ch;
+        unsigned char prev_ch=0;
+        unsigned char iswin=0;
         unsigned char msgparsestate;
         unsigned char cksum=0;
         unsigned char seqnum=0;
@@ -592,6 +693,14 @@ void main(void)
         LED_OFF;
         sei();
         msgparsestate=MSG_IDLE;
+	// default values:
+	CONFIG_PARAM_SW_MINOR=D_CONFIG_PARAM_SW_MINOR;
+	CONFIG_PARAM_SW_MAJOR=D_CONFIG_PARAM_SW_MAJOR;
+        if (eeprom_read_byte((uint8_t *)EEPROM_MAGIC) == 20){
+                // ok magic number matches accept values
+                CONFIG_PARAM_SW_MINOR=eeprom_read_byte(EEPROM_MINOR);
+                CONFIG_PARAM_SW_MAJOR=eeprom_read_byte(EEPROM_MAJOR);
+        }
         while(1){
                 ch=uart_getchar();
                 // parse message according to appl. note AVR068 table 3-1:
@@ -600,6 +709,30 @@ void main(void)
                         cksum = ch^0;
                         continue;
                 }
+		// the special avrusb500 terminal mode.
+		// just connect a serial terminal and press enter twice.
+                if (msgparsestate==MSG_IDLE && (ch == '\r'||ch == '\n')){
+			i++;
+			if(iswin ==0 && ch=='\n' && prev_ch== '\r'){
+				// terminal sends \r\n for new line
+				iswin=1;
+			}
+			prev_ch=ch;
+			if (iswin){
+				// wait for \r\n \r\n
+				if (i==4){
+					terminalmode(1);
+					i=0;
+				}
+			}else{
+				// Linux wait for \n\n
+				if (i==2){
+					terminalmode(0);
+					i=0;
+				}
+			}
+			continue;
+		}
                 if (msgparsestate==MSG_WAIT_SEQNUM){
                         seqnum=ch;
                         cksum^=ch;
@@ -652,7 +785,9 @@ void main(void)
                 msgparsestate=MSG_IDLE;
                 msglen=0;
                 seqnum=0;
+		prev_ch=0;
                 i=0;
         }
+	return(0);
 }
 
