@@ -5,28 +5,23 @@
 * Copyright: GPL
 **********************************************/
 
-#include <avr/signal.h>
+//#include <avr/signal.h>
 #include <avr/interrupt.h>
 #include <avr/pgmspace.h>
 #include <avr/io.h>
 #include <avr/wdt.h>
+#define F_CPU 3686400UL  // 3.6864 MHz
+#include <avr/delay.h>
 #include "avr_compat.h"
 
 void delay_ms(unsigned int ms)
-/* delay for about a minimum of <ms> */
-/* with a 4Mhz clock */
+/* delay for a minimum of <ms> */
 {
-        uint8_t inner,inner1;
-        while (ms) {
-                inner = 150;
-                while (inner) {
-                        inner--;
-                        inner1 = 10;
-                        while (inner1) {
-                                inner1--;
-                                asm("nop");
-                        }
-                }
+        // we use a calibrated macro. This is more
+        // accurate and not so much compiler dependent
+        // as self made code.
+        while(ms){
+                _delay_loop_2(240.0);
                 ms--;
         }
 }
